@@ -8,6 +8,8 @@
     email                : ojschmidt@kde.org
     copyright            : (C) 2008 by Matthew Woehlke
     email                : mw_triad@users.sourceforge.net
+    copyright              (C) 2010 Sebastian Sauer
+    email                  sebsauer@kdab.com
  ***************************************************************************/
 
 /***************************************************************************
@@ -65,46 +67,44 @@
 #include "kmagselrect.h"
 
 
-#define havesetCheckedState
-
 KmagApp::KmagApp(QWidget* , const char* name)
-  : KXmlGuiWindow(0, Qt::WStyle_MinMax | Qt::WType_TopLevel | Qt::WDestructiveClose | Qt::WStyle_ContextHelp | Qt::WStyle_StaysOnTop | Qt::WindowCloseButtonHint),
-    m_defaultMouseCursorType(2)
+  : KXmlGuiWindow(0) // Qt::WStyle_MinMax | Qt::WType_TopLevel | Qt::WDestructiveClose | Qt::WStyle_ContextHelp | Qt::WindowCloseButtonHint | Qt::WStyle_StaysOnTop
+  , m_defaultMouseCursorType(2)
 {
-  setObjectName(name);
+  setObjectName( QLatin1String( name ) );
+  setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
   config=KGlobal::config();
 
-  zoomArrayString << "5:1" << "2:1" << "1:1" << "1:1.5" << "1:2" << "1:3" << "1:4" << "1:5"
-    << "1:6" << "1:7" << "1:8" << "1:12" << "1:16" << "1:20";
+  zoomArrayString << QLatin1String( "5:1" ) << QLatin1String( "2:1" ) << QLatin1String( "1:1" ) << QLatin1String( "1:1.5" ) << QLatin1String( "1:2" ) << QLatin1String( "1:3" ) << QLatin1String( "1:4" ) << QLatin1String( "1:5" )
+    << QLatin1String( "1:6" ) << QLatin1String( "1:7" ) << QLatin1String( "1:8" ) << QLatin1String( "1:12" ) << QLatin1String( "1:16" ) << QLatin1String( "1:20" );
 
-  // Is there a better way to initialize a vector array?
-  zoomArray.push_back(0.2); zoomArray.push_back(0.5); zoomArray.push_back(1.0);
-  zoomArray.push_back(1.5); zoomArray.push_back(2.0); zoomArray.push_back(3.0);
-  zoomArray.push_back(4.0); zoomArray.push_back(5.0); zoomArray.push_back(6.0); zoomArray.push_back(7.0);
-  zoomArray.push_back(8.0); zoomArray.push_back(12.0); zoomArray.push_back(16.0); zoomArray.push_back(20.0);
+  zoomArray << 0.2 <<0.5 << 1.0;
+  zoomArray << 1.5 << 2.0 << 3.0;
+  zoomArray << 4.0 << 5.0 << 6.0 << 7.0;
+  zoomArray << 8.0 << 12.0 << 16.0 << 20.0;
 
   fpsArrayString << i18nc("Zoom at very low", "&Very Low") << i18nc("Zoom at low", "&Low") << i18nc("Zoom at medium", "&Medium") << i18nc("Zoom at high", "&High") << i18nc("Zoom at very high", "V&ery High");
 
-  fpsArray.push_back(2); // very low
-  fpsArray.push_back(6); // low
-  fpsArray.push_back(10); // medium
-  fpsArray.push_back(15); // high
-  fpsArray.push_back(25); // very high
+  fpsArray.append(2); // very low
+  fpsArray.append(6); // low
+  fpsArray.append(10); // medium
+  fpsArray.append(15); // high
+  fpsArray.append(25); // very high
 
   colorArrayString << i18nc("No color-blindness simulation, i.e. 'normal' vision", "&Normal") << i18n("&Protanopia") << i18n("&Deuteranopia") << i18n("&Tritanopia") << i18n("&Achromatopsia");
 
-  colorArray.push_back(0);
-  colorArray.push_back(1);
-  colorArray.push_back(2);
-  colorArray.push_back(3);
-  colorArray.push_back(4);
+  colorArray.append(0);
+  colorArray.append(1);
+  colorArray.append(2);
+  colorArray.append(3);
+  colorArray.append(4);
 
   rotationArrayString << i18n("&No Rotation (0 Degrees)") << i18n("&Left (90 Degrees)") << i18n("&Upside Down (180 Degrees)") << i18n("&Right (270 Degrees)");
 
-  rotationArray.push_back(0); // no rotation
-  rotationArray.push_back(90); // left
-  rotationArray.push_back(180); // upside down
-  rotationArray.push_back(270); // right
+  rotationArray.append(0); // no rotation
+  rotationArray.append(90); // left
+  rotationArray.append(180); // upside down
+  rotationArray.append(270); // right
 
   // call inits to invoke all other construction parts
   initView();
@@ -133,15 +133,15 @@ KmagApp::~KmagApp()
 
 void KmagApp::initActions()
 {
-  fileNewWindow = actionCollection()->addAction("new_window");
-  fileNewWindow->setIcon(KIcon("window-new"));
+  fileNewWindow = actionCollection()->addAction(QLatin1String( "new_window" ));
+  fileNewWindow->setIcon(KIcon(QLatin1String( "window-new" )));
   fileNewWindow->setText(i18n("New &Window"));
   connect(fileNewWindow, SIGNAL(triggered(bool) ), SLOT(slotFileNewWindow()));
   fileNewWindow->setShortcuts(KStandardShortcut::openNew());
   fileNewWindow->setToolTip(i18n("Open a new KMagnifier window"));
 
-  refreshSwitch = actionCollection()->addAction("start_stop_refresh");
-  refreshSwitch->setIcon(KIcon("process-stop"));
+  refreshSwitch = actionCollection()->addAction(QLatin1String( "start_stop_refresh" ));
+  refreshSwitch->setIcon(KIcon(QLatin1String( "process-stop" )));
   refreshSwitch->setText(i18n("&Stop"));
   connect(refreshSwitch, SIGNAL(triggered(bool) ), SLOT(slotToggleRefresh()));
   refreshSwitch->setShortcuts(KStandardShortcut::reload());
@@ -150,8 +150,8 @@ void KmagApp::initActions()
   updating of the display. Stopping the update will zero the processing power\
   required (CPU usage)"));
 
-  m_pSnapshot = actionCollection()->addAction("snapshot");
-  m_pSnapshot->setIcon(KIcon("ksnapshot"));
+  m_pSnapshot = actionCollection()->addAction(QLatin1String( "snapshot" ));
+  m_pSnapshot->setIcon(KIcon(QLatin1String( "ksnapshot" )));
   m_pSnapshot->setText(i18n("&Save Snapshot As..."));
   connect(m_pSnapshot, SIGNAL(triggered(bool) ), SLOT(saveZoomPixmap()));
   m_pSnapshot->setShortcuts(KStandardShortcut::save());
@@ -169,66 +169,62 @@ void KmagApp::initActions()
   m_pCopy->setWhatsThis(i18n("Click on this button to copy the current zoomed view to the clipboard which you can paste in other applications."));
   m_pCopy->setToolTip(i18n("Copy zoomed image to clipboard"));
 
-  m_pShowMenu = new KToggleAction(KIcon("show-menu"), i18n("Show &Menu"), this);
-  actionCollection()->addAction("show_menu", m_pShowMenu);
-  connect(m_pShowMenu, SIGNAL(triggered(bool)), SLOT(slotShowMenu()));
-  m_pShowMenu->setShortcut(Qt::CTRL+Qt::Key_M);
-  #ifdef havesetCheckedState
-  m_pShowMenu->setCheckedState(KGuiItem(i18n("Hide &Menu")));
-  #endif
-  m_pShowMainToolBar = new KToggleAction(i18n("Show Main &Toolbar"), this);
-  actionCollection()->addAction("show_mainToolBar", m_pShowMainToolBar);
-  connect(m_pShowMainToolBar, SIGNAL(triggered(bool)), SLOT(slotShowMainToolBar()));
-  #ifdef havesetCheckedState
-  m_pShowMainToolBar->setCheckedState(KGuiItem(i18n("Hide Main &Toolbar")));
-  #endif
-  m_pShowViewToolBar = new KToggleAction(i18n("Show &View Toolbar"), this);
-  actionCollection()->addAction("show_viewToolBar", m_pShowViewToolBar);
-  connect(m_pShowViewToolBar, SIGNAL(triggered(bool)), SLOT(slotShowViewToolBar()));
-  #ifdef havesetCheckedState
-  m_pShowViewToolBar->setCheckedState(KGuiItem(i18n("Hide &View Toolbar")));
-  #endif
-  m_pShowSettingsToolBar = new KToggleAction(i18n("Show &Settings Toolbar"), this);
-  actionCollection()->addAction("show_settingsToolBar", m_pShowSettingsToolBar);
-  connect(m_pShowSettingsToolBar, SIGNAL(triggered(bool)), SLOT(slotShowSettingsToolBar()));
-  #ifdef havesetCheckedState
-  m_pShowSettingsToolBar->setCheckedState(KGuiItem(i18n("Hide &Settings Toolbar")));
-  #endif
+  m_pShowMenu = KStandardAction::showMenubar(this, SLOT(slotShowMenu()), actionCollection());
 
-  m_modeFollowMouse = new KToggleAction(KIcon("followmouse"), i18n("&Follow Mouse Mode"), this);
-  actionCollection()->addAction("mode_followmouse", m_modeFollowMouse);
+  setStandardToolBarMenuEnabled(true);
+
+  m_modeFollowMouse = new KToggleAction(KIcon(QLatin1String( "followmouse" )), i18n("&Follow Mouse Mode"), this);
+  actionCollection()->addAction(QLatin1String( "mode_followmouse" ), m_modeFollowMouse);
   connect(m_modeFollowMouse, SIGNAL(triggered(bool)), SLOT(slotModeFollowMouse()));
   m_modeFollowMouse->setShortcut(Qt::Key_F1);
+  m_modeFollowMouse->setIconText(i18n("Mouse"));
   m_modeFollowMouse->setToolTip(i18n("Magnify around the mouse cursor"));
   m_modeFollowMouse->setWhatsThis(i18n("If selected, the area around the mouse cursor is magnified"));
 
-  m_modeSelWin = new KToggleAction(KIcon("window"), i18n("Se&lection Window Mode"), this);
-  actionCollection()->addAction("mode_selectionwindow", m_modeSelWin);
+  m_modeFollowFocus = new KToggleAction(KIcon(QLatin1String( "view-restore" )), i18n("&Follow Focus Mode"), this);
+  actionCollection()->addAction(QLatin1String( "mode_followfocus" ), m_modeFollowFocus);
+  connect(m_modeFollowFocus, SIGNAL(triggered(bool)), SLOT(slotModeFollowFocus()));
+  m_modeFollowFocus->setShortcut(Qt::Key_F2);
+  m_modeFollowFocus->setIconText(i18n("Focus"));
+  m_modeFollowFocus->setToolTip(i18n("Magnify around the keyboard focus"));
+  m_modeFollowFocus->setWhatsThis(i18n("If selected, the area around the keyboard cursor is magnified"));
+
+  m_modeSelWin = new KToggleAction(KIcon(QLatin1String( "window" )), i18n("Se&lection Window Mode"), this);
+  actionCollection()->addAction(QLatin1String( "mode_selectionwindow" ), m_modeSelWin);
   connect(m_modeSelWin, SIGNAL(triggered(bool)), SLOT(slotModeSelWin()));
-  m_modeSelWin->setShortcut(Qt::Key_F2);
+  m_modeSelWin->setShortcut(Qt::Key_F3);
+  m_modeSelWin->setIconText(i18n("Window"));
   m_modeSelWin->setToolTip(i18n("Show a window for selecting the magnified area"));
 
-  m_modeWholeScreen = new KToggleAction(KIcon("view-fullscreen"), i18n("&Whole Screen Mode"), this);
-  actionCollection()->addAction("mode_wholescreen", m_modeWholeScreen);
+  m_modeWholeScreen = new KToggleAction(KIcon(QLatin1String( "view-fullscreen" )), i18n("&Whole Screen Mode"), this);
+  actionCollection()->addAction(QLatin1String( "mode_wholescreen" ), m_modeWholeScreen);
   connect(m_modeWholeScreen, SIGNAL(triggered(bool)), SLOT(slotModeWholeScreen()));
-  m_modeWholeScreen->setShortcut(Qt::Key_F3);
+  m_modeWholeScreen->setShortcut(Qt::Key_F4);
+  m_modeWholeScreen->setIconText(i18n("Screen"));
   m_modeWholeScreen->setToolTip(i18n("Magnify the whole screen"));
   m_modeWholeScreen->setWhatsThis(i18n("Click on this button to fit the zoom view to the zoom window."));
 
-  m_hideCursor = new KToggleAction(KIcon("hidemouse"), i18n("Hide Mouse &Cursor"), this);
-  actionCollection()->addAction("hidecursor", m_hideCursor);
+  m_hideCursor = new KToggleAction(KIcon(QLatin1String( "hidemouse" )), i18n("Hide Mouse &Cursor"), this);
+  actionCollection()->addAction(QLatin1String( "hidecursor" ), m_hideCursor);
   connect(m_hideCursor, SIGNAL(triggered(bool)), SLOT(slotToggleHideCursor()));
-  m_hideCursor->setShortcut(Qt::Key_F4);
-  #ifdef havesetCheckedState
+  m_hideCursor->setShortcut(Qt::Key_F5);
+  #ifdef havesetCheckedStatef
   m_hideCursor->setCheckedState(KGuiItem(i18n("Show Mouse &Cursor")));
   #endif
+  m_hideCursor->setIconText(i18n("Hide"));
   m_hideCursor->setToolTip(i18n("Hide the mouse cursor"));
 
+  m_staysOnTop = new KToggleAction(KIcon(QLatin1String( "go-top" )), i18n("Stays On Top"), this);
+  actionCollection()->addAction(QLatin1String( "staysontop" ), m_staysOnTop);
+  connect(m_staysOnTop, SIGNAL(triggered(bool)), SLOT(slotStaysOnTop()));
+  m_staysOnTop->setShortcut(Qt::Key_F6);
+  m_staysOnTop->setToolTip(i18n("The KMagnifier Window stays on top of other windows."));
+
   m_pZoomIn = actionCollection()->addAction(KStandardAction::ZoomIn, this, SLOT(zoomIn()));
   m_pZoomIn->setWhatsThis(i18n("Click on this button to <b>zoom-in</b> on the selected region."));
 
   m_pZoomBox = new KSelectAction(i18n("&Zoom"), this);
-  actionCollection()->addAction("zoom", m_pZoomBox);
+  actionCollection()->addAction(QLatin1String( "zoom" ), m_pZoomBox);
   m_pZoomBox->setItems(zoomArrayString);
   m_pZoomBox->setWhatsThis(i18n("Select the zoom factor."));
   m_pZoomBox->setToolTip(i18n("Zoom factor"));
@@ -237,7 +233,7 @@ void KmagApp::initActions()
   m_pZoomOut->setWhatsThis(i18n("Click on this button to <b>zoom-out</b> on the selected region."));
 
   m_pRotationBox = new KSelectAction(i18n("&Rotation"),this);
-  actionCollection()->addAction("rotation", m_pRotationBox);
+  actionCollection()->addAction(QLatin1String( "rotation" ), m_pRotationBox);
   m_pRotationBox->setItems(rotationArrayString);
   m_pRotationBox->setWhatsThis(i18n("Select the rotation degree."));
   m_pRotationBox->setToolTip(i18n("Rotation degree"));
@@ -248,26 +244,25 @@ void KmagApp::initActions()
   m_toolConf = actionCollection()->addAction(KStandardAction::ConfigureToolbars, this, SLOT(slotEditToolbars()));
 
   m_pFPSBox = new KSelectAction(i18n("&Refresh"),this);
-  actionCollection()->addAction("fps_selector", m_pFPSBox);
+  actionCollection()->addAction(QLatin1String( "fps_selector" ), m_pFPSBox);
   m_pFPSBox->setItems(fpsArrayString);
   m_pFPSBox->setWhatsThis(i18n("Select the refresh rate. The higher the rate, the more computing power (CPU) will be needed."));
   m_pFPSBox->setToolTip(i18n("Refresh rate"));
 
   m_pColorBox = new KSelectAction(i18nc("Color-blindness simulation mode", "&Color"),this);
-  actionCollection()->addAction("color_mode", m_pColorBox);
+  actionCollection()->addAction(QLatin1String( "color_mode" ), m_pColorBox);
   m_pColorBox->setItems(colorArrayString);
   m_pColorBox->setWhatsThis(i18n("Select a mode to simulate various types of color-blindness."));
   m_pColorBox->setToolTip(i18n("Color-blindness Simulation Mode"));
 
-  createGUI();
+  setupGUI();
 }
 
 void KmagApp::initView()
 {
   m_zoomView = new KMagZoomView( this, "ZoomView" );
-  m_zoomView->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, m_zoomView->sizePolicy().hasHeightForWidth() ) );
-  m_zoomView->setFrameShape( QFrame::StyledPanel );
-  m_zoomView->setFrameShadow( QFrame::Raised );
+//m_zoomView->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, m_zoomView->sizePolicy().hasHeightForWidth() ) );
+  m_zoomView->setFrameShape( QFrame::NoFrame );
 
   setCentralWidget(m_zoomView);
 }
@@ -330,25 +325,16 @@ void KmagApp::saveOptions()
   cg.writeEntry("ColorIndex", m_colorIndex);
   cg.writeEntry("SelRect", m_zoomView->getSelRectPos());
   cg.writeEntry("ShowMouse", m_zoomView->getShowMouseType());
+  cg.writeEntry("StaysOnTop", m_staysOnTop->isChecked());
 
   if (m_modeFollowMouse->isChecked())
      cg.writeEntry("Mode", "followmouse");
+  else if (m_modeFollowFocus->isChecked())
+     cg.writeEntry("Mode", "followfocus");
   else if (m_modeWholeScreen->isChecked())
      cg.writeEntry("Mode", "wholescreen");
   else if (m_modeSelWin->isChecked())
      cg.writeEntry("Mode", "selectionwindow");
-
-  cg.writeEntry("ShowMenu", m_pShowMenu->isChecked());
-  cg.writeEntry("ShowMainToolBar", m_pShowMainToolBar->isChecked());
-  cg.writeEntry("ShowViewToolBar", m_pShowViewToolBar->isChecked());
-  cg.writeEntry("ShowSettingsToolBar", m_pShowSettingsToolBar->isChecked());
-
-  KConfigGroup mainToolBarGroup(config,"Main ToolBar");
-  toolBar("mainToolBar")->saveSettings( mainToolBarGroup );
-  KConfigGroup viewToolBarGroup(config,"View ToolBar" );
-  toolBar("viewToolBar")->saveSettings( viewToolBarGroup );
-  KConfigGroup settingsToolbarGroup(config,"Settings ToolBar" );
-  toolBar("settingsToolBar")->saveSettings( settingsToolbarGroup );
 }
 
 
@@ -394,10 +380,12 @@ void KmagApp::readOptions()
   emit updateColorIndex(colorIndex);
 
   QString mode = cg.readEntry("Mode", "followmouse");
-  if (mode == "wholescreen")
+  if (mode == QLatin1String( "wholescreen" ))
     slotModeWholeScreen();
-  else if (mode == "selectionwindow")
+  else if (mode == QLatin1String( "selectionwindow" ))
     slotModeSelWin();
+  else if (mode == QLatin1String( "followfocus" ))
+    slotModeFollowFocus();
   else
     slotModeFollowMouse();
 
@@ -406,35 +394,13 @@ void KmagApp::readOptions()
 
   m_mouseCursorType = cg.readEntry("ShowMouse", m_defaultMouseCursorType);
   m_zoomView->showMouse(m_mouseCursorType);
-  if(m_mouseCursorType)
-    m_hideCursor->setChecked(false);
-  else
-    m_hideCursor->setChecked(true);
+  m_hideCursor->setChecked(!m_mouseCursorType);
 
-  KConfigGroup settingsToolbarGroup(config,"Settings ToolBar");
-  if( settingsToolbarGroup.exists() )
-      toolBar("settingsToolBar")->applySettings(settingsToolbarGroup );
+  m_staysOnTop->setChecked(cg.readEntry("StaysOnTop", true));
+  slotStaysOnTop();
 
-  KConfigGroup mainToolbarGroup(config,"Main ToolBar");
-  if( mainToolbarGroup.exists() )
-      toolBar("mainToolBar")->applySettings( mainToolbarGroup );
-
-  KConfigGroup viewToolbarGroup(config,"View ToolBar" );
-  if( viewToolbarGroup.exists() )
-      toolBar("viewToolBar")->applySettings( viewToolbarGroup );
-
-  KConfigGroup generalOptionGroup(config,"General Options");
-  m_pShowMenu->setChecked( generalOptionGroup.readEntry("ShowMenu", true));
-  slotShowMenu();
-
-  m_pShowMainToolBar->setChecked(generalOptionGroup.readEntry("ShowMainToolBar", false));
-  slotShowMainToolBar();
-
-  m_pShowViewToolBar->setChecked(generalOptionGroup.readEntry("ShowViewToolBar", true));
-  slotShowViewToolBar();
-
-  m_pShowSettingsToolBar->setChecked(generalOptionGroup.readEntry("ShowSettingsToolBar", true));
-  slotShowSettingsToolBar();
+  // XMLGui has already read and set up the menuBar for us
+  m_pShowMenu->setChecked(menuBar()->isVisible());
 }
 
 bool KmagApp::queryClose()
@@ -457,9 +423,9 @@ void KmagApp::contextMenuEvent ( QContextMenuEvent * e )
 {
  // show popup
  KXMLGUIFactory *factory = this->factory();
- QMenu *popup = (QMenu *)factory->container("mainPopUp",this);
+ QMenu *popup = (QMenu *)factory->container(QLatin1String( "mainPopUp" ),this);
  if (popup != 0)
-   popup->popup(e->globalPos(), 0);
+   popup->popup(e->globalPos());
  e->accept();
 }
 
@@ -536,7 +502,7 @@ void KmagApp::setZoomIndex(int index)
   }
 
   // signal change in zoom value
-  emit updateZoomValue(zoomArray[m_zoomIndex]);
+  emit updateZoomValue(zoomArray.at(m_zoomIndex));
 }
 
 /**
@@ -556,7 +522,7 @@ void KmagApp::setRotationIndex(int index)
   }
 
   // signal change in zoom value
-  emit updateRotationValue(rotationArray[m_rotationIndex]);
+  emit updateRotationValue(rotationArray.at(m_rotationIndex));
 }
 
 /**
@@ -576,7 +542,7 @@ void KmagApp::setFPSIndex(int index)
   }
 
   // signal change in fps value
-  emit updateFPSValue(fpsArray[m_fpsIndex]);
+  emit updateFPSValue(fpsArray.at(m_fpsIndex));
 }
 
 /**
@@ -596,7 +562,7 @@ void KmagApp::setColorIndex(int index)
   }
 
   // signal change in fps value
-  emit updateColorValue(colorArray[m_colorIndex]);
+  emit updateColorValue(colorArray.at(m_colorIndex));
 }
 
 /**
@@ -623,7 +589,7 @@ void KmagApp::saveZoomPixmap()
 #ifdef __GNUC__
 #warning "kde4: port KImageIO::type \n";
 #endif
-      if(!tempFile.open() || !m_zoomView->getPixmap().save(tempFile.fileName(),"png"/*, KImageIO::type(url.fileName()).latin1()*/)) {
+      if(!tempFile.open() || !m_zoomView->getImage().save(tempFile.fileName(),"png"/*, KImageIO::type(url.fileName()).latin1()*/)) {
         KMessageBox::error(0, i18n("Unable to save temporary file (before uploading to the network file you specified)."),
                           i18n("Error Writing File"));
       } else {
@@ -632,7 +598,7 @@ void KmagApp::saveZoomPixmap()
                             i18n("Error Writing File"));
         } else {
           KMessageBox::information(0, i18n("Current zoomed image saved to\n%1", url.prettyUrl()),
-                              i18n("Information"), "save_confirm");
+                              i18n("Information"), QLatin1String( "save_confirm" ));
         }
       }
 
@@ -640,12 +606,12 @@ void KmagApp::saveZoomPixmap()
 #ifdef __GNUC__
 #warning "kde4 : port KImageIO::type(...) \n";
 #endif
-      if(!m_zoomView->getPixmap().save(url.path(), "png"/*KImageIO::type(url.fileName()).latin1()*/)) {
+      if(!m_zoomView->getImage().save(url.path(), "png"/*KImageIO::type(url.fileName()).latin1()*/)) {
         KMessageBox::error(0, i18n("Unable to save file. Please check if you have permission to write to the directory."),
                             i18n("Error Writing File"));
       } else {
         KMessageBox::information(0, i18n("Current zoomed image saved to\n%1", url.prettyUrl()),
-                                i18n("Information"), "save_confirm");
+                                i18n("Information"), QLatin1String( "save_confirm" ));
       }
     }
   }
@@ -659,11 +625,11 @@ void KmagApp::slotToggleRefresh()
 {
   m_zoomView->toggleRefresh();
   if(m_zoomView->getRefreshStatus()) {
-    refreshSwitch->setIcon(KIcon("process-stop"));
+    refreshSwitch->setIcon(KIcon(QLatin1String( "process-stop" )));
     refreshSwitch->setText(i18n("Stop"));
     refreshSwitch->setToolTip(i18n("Click to stop window update"));
   } else {
-    refreshSwitch->setIcon(KIcon("reload"));
+    refreshSwitch->setIcon(KIcon(QLatin1String( "system-run" )));
     refreshSwitch->setText(i18nc("Start updating the window", "Start"));
     refreshSwitch->setToolTip(i18n("Click to start window update"));
   }
@@ -672,11 +638,13 @@ void KmagApp::slotToggleRefresh()
 
 void KmagApp::slotModeWholeScreen()
 {
-  m_zoomView->setSelRectPos(QRect (0, 0, QApplication::desktop()->width(), QApplication::desktop()->height()));
   m_zoomView->followMouse(false);
+  m_zoomView->followFocus(false);
+  m_zoomView->setSelRectPos(QRect (0, 0, QApplication::desktop()->width(), QApplication::desktop()->height()));
   m_zoomView->showSelRect(false);
   m_zoomView->setFitToWindow (false);
   m_modeFollowMouse->setChecked(false);
+  m_modeFollowFocus->setChecked(false);
   m_modeWholeScreen->setChecked(true);
   m_modeSelWin->setChecked(false);
 }
@@ -685,9 +653,11 @@ void KmagApp::slotModeWholeScreen()
 void KmagApp::slotModeSelWin()
 {
   m_zoomView->followMouse(false);
+  m_zoomView->followFocus(false);
   m_zoomView->showSelRect(true);
   m_zoomView->setFitToWindow (false);
   m_modeFollowMouse->setChecked(false);
+  m_modeFollowFocus->setChecked(false);
   m_modeWholeScreen->setChecked(false);
   m_modeSelWin->setChecked(true);
 }
@@ -696,9 +666,24 @@ void KmagApp::slotModeSelWin()
 void KmagApp::slotModeFollowMouse()
 {
   m_zoomView->followMouse(true);
+  m_zoomView->followFocus(false);
   m_zoomView->showSelRect(false);
   m_zoomView->setFitToWindow (true);
   m_modeFollowMouse->setChecked(true);
+  m_modeFollowFocus->setChecked(false);
+  m_modeWholeScreen->setChecked(false);
+  m_modeSelWin->setChecked(false);
+}
+
+
+void KmagApp::slotModeFollowFocus()
+{
+  m_zoomView->followMouse(false);
+  m_zoomView->followFocus(true);
+  m_zoomView->showSelRect(false);
+  m_zoomView->setFitToWindow (true);
+  m_modeFollowMouse->setChecked(false);
+  m_modeFollowFocus->setChecked(true);
   m_modeWholeScreen->setChecked(false);
   m_modeSelWin->setChecked(false);
 }
@@ -710,6 +695,16 @@ void KmagApp::slotToggleHideCursor()
 }
 
 
+void KmagApp::slotStaysOnTop()
+{
+  if (m_staysOnTop->isChecked())
+      setWindowFlags( windowFlags() | Qt::WindowStaysOnTopHint );
+  else
+      setWindowFlags( windowFlags() & !Qt::WindowStaysOnTopHint );
+  show();
+}
+
+
 void KmagApp::slotFileNewWindow()
 {
   KmagApp *new_window= new KmagApp();
@@ -733,7 +728,7 @@ void KmagApp::slotFilePrint()
     toggled = true;
   }
 
-  const QPixmap pixmap(m_zoomView->getPixmap());
+  const QImage pixmap(m_zoomView->getImage());
 
   // use some AI to get the best orientation
   if(pixmap.width() > pixmap.height()) {
@@ -753,7 +748,7 @@ void KmagApp::slotFilePrint()
       return;
     }
     // draw the pixmap
-    paint.drawPixmap(0, 0, pixmap);
+    paint.drawImage(0, 0, pixmap);
     // end the painting
     paint.end();
   }
@@ -791,7 +786,7 @@ void KmagApp::slotFileQuit()
 void KmagApp::copyToClipBoard()
 {
   QClipboard *cb=KApplication::clipboard();
-  cb->setPixmap(m_zoomView->getPixmap());
+  cb->setPixmap(QPixmap::fromImage(m_zoomView->getImage()));
 }
 
 void KmagApp::slotShowMenu()
@@ -806,50 +801,6 @@ void KmagApp::slotShowMenu()
   {
     menuBar()->show();
   }
-
-
-}
-
-void KmagApp::slotShowMainToolBar()
-{
-  ///////////////////////////////////////////////////////////////////
-  // turn mainToolbar on or off
-  if(!m_pShowMainToolBar->isChecked())
-  {
-    toolBar("mainToolBar")->hide();
-  }
-  else
-  {
-    toolBar("mainToolBar")->show();
-  }
-}
-
-void KmagApp::slotShowViewToolBar()
-{
-  ///////////////////////////////////////////////////////////////////
-  // turn viewToolbar on or off
-  if(!m_pShowViewToolBar->isChecked())
-  {
-    toolBar("viewToolBar")->hide();
-  }
-  else
-  {
-    toolBar("viewToolBar")->show();
-  }
-}
-
-void KmagApp::slotShowSettingsToolBar()
-{
-  ///////////////////////////////////////////////////////////////////
-  // turn viewToolbar on or off
-  if(!m_pShowSettingsToolBar->isChecked())
-  {
-    toolBar("settingsToolBar")->hide();
-  }
-  else
-  {
-    toolBar("settingsToolBar")->show();
-  }
 }
 
 void KmagApp::slotConfKeys()
@@ -862,7 +813,7 @@ void KmagApp::slotEditToolbars()
   KConfigGroup cg( KGlobal::config(), "MainWindow" );
   saveMainWindowSettings( cg );
   KEditToolBar dlg( actionCollection() );
-  connect( &dlg, SIGNAL( newToolbarConfig() ), this, SLOT( slotNewToolbarConfig() ) );
+  connect( &dlg, SIGNAL( newToolBarConfig() ), this, SLOT( slotNewToolbarConfig() ) );
   if ( dlg.exec() )
     createGUI();
 }
